# -*- coding: utf-8 -*-
"""
Created on Mon May  8 13:19:02 2017

@author: yoann
"""

import numpy
import queue
from sklearn import cluster
from sklearn.base import BaseEstimator, ClassifierMixin

from matplotlib.pyplot import cm
from matplotlib import pyplot
import cProfile

import cython



cdef public class KMeansForest(BaseEstimator, ClassifierMixin):
    """
        A KMeans decision trees ensemble.

        :param n_estimators: int
            The number of trees to fit
        :param max_features: int, float (default=0.75)
            The number of features to consider when splitting the nodes
            If int, consider max_features at each split
            If float, max_features is a percentage and int(n_features * self.max_features) are considered at each split
        :param propagation: string
            The method of propagation used when predicting an example class.
            If 'used', propagate in the subspace used at each node
            If 'all', propagate in the entire space

        Attributes
        ----------
        estimators : array
            The array of fitted estimators

        Properties
        ----------
        node_count : int
            The total number of nodes of the estimators
    """
    cdef int n_estimators
    cdef double max_features

    def __init__(self, n_estimators=10, max_features=0.75, propagation='used'):
        self.n_estimators = n_estimators
        self.estimators = numpy.array([])
        self.max_features = max_features
        self.propagation = propagation

    cpdef reset(self):
        self.estimators = numpy.array([])

    cpdef fit(self, examples, classes):
        self.reset()
        for estimator_index in range(self.n_estimators):
            self.estimators = numpy.append(self.estimators, [KMeansDecisionTree(self.max_features, self.propagation)])
            self.estimators[estimator_index].fit(examples, classes)

    cpdef predict(self, examples):
        if len(self.estimators) == 0:
            raise ValueError('Estimator not fitted')

        predictions = numpy.array([tree.predict(examples) for tree in self.estimators], dtype='int')
        counts = [numpy.bincount(predictions[:, i]) for i in range(predictions.shape[1])]
        return numpy.array([numpy.argmax(count_example) for count_example in counts])

    @property
    cpdef node_count(self):
        return sum([tree.node_count for tree in self.estimators])

    cpdef __repr__(self):
        return super().__repr__()

